package net.orfjackal.util;

import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.List;

/**
 * Library class of useful file operations.
 *
 * @author Esko Luontola
 */
public class FileTools {

    /**
     * Reads the given file to a string.
     *
     * @param file the file to be read.
     * @return the contents of the file as a string.
     * @throws java.io.IOException  if the file can not be read.
     * @throws NullPointerException if file is null.
     */
    @NotNull
    public static String readFile(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException();
        }
        Reader in = new FileReader(file);
        StringBuilder sb = new StringBuilder((int) file.length());
        int i;
        while ((i = in.read()) != -1) {
            sb.append((char) i);
        }
        in.close();
        return sb.toString();
    }

    /**
     * Adds recursively all files in the specified directory to a list. Will not add duplicates to the result list.
     *
     * @param result where all files that were found will be added.
     * @param file   the directory which to read recursively. If it is a regular file, it will be added to the result
     *               list.
     * @throws NullPointerException if any parameter is null.
     */
    public static void recursiceAdd(List<File> result, File file) {
        if (file.isDirectory()) {
            File[] subdirFiles = file.listFiles();
            if (subdirFiles == null) {
                System.err.println("Unable to read directory: " + file);
                return;
            }
            for (File f : subdirFiles) {
                recursiceAdd(result, f);
            }
        } else if (!result.contains(file)) {
            result.add(file);
        }
    }
}
