package net.orfjackal.util.commandline;

import net.orfjackal.util.FileTools;

import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Standard argument for handling file name parameters. If the parameters are directories, this will dig recursively
 * into them. If one of the parameters does not exist, a ParseException will be thrown.
 *
 * @author Esko Luontola
 */
public abstract class FileArgument extends Argument {

    public FileArgument(String name, boolean required) {
        super(name, required);
    }

    public FileArgument(String name, boolean required, boolean multiple) {
        super(name, required, multiple);
    }

    public void onSpecified(String[] args) throws ParseException {
        List<File> files = new ArrayList<File>();
        for (String s : args) {
            File file = new File(s);
            if (!file.exists()) {
                throw new ParseException("No such file or directory: " + file, 0);
            }
            FileTools.recursiceAdd(files, file);
        }
        File[] fileArray = files.toArray(new File[files.size()]);
        Arrays.sort(fileArray);
        onSpecified(fileArray);
    }

    public abstract void onSpecified(File[] files) throws ParseException;
}
