// Tietorakenteiden harjoitusty (syksy 2004), Esko Luontola
// Tietoliikenneverkon reititysalgoritmien vertailu, "Sim Router"

/**
 * Reitittimiss kytettv puskuroiva vastaanottoportti, joka saa pakettinsa 
 * toisen reitittimen lhetysportilta. Tm saa suoritusvuoronsa lhetysportilta
 * ja siirt edelleen suoritusvuoron vastaanottavalle reitittimelle.
 */
public class RecievePort {
	
	/**
	 * Reititin jonne tmn kautta kulkevat paketit menevt.
	 */
	private final Router to;
	
	/**
	 * Puskuri reitityst odottaville paketeille.
	 */
	private final PacketBuffer buffer;
	
	/**
	 * Luo vastaanottoportin reitittimelle to, puskurikoolla bufferSize.
	 */
	public RecievePort(Router to, int bufferSize) {
		if (to == null) {
			throw new NullPointerException();
		}
		this.to = to;
		this.buffer = new PacketBuffer(bufferSize);
	}
	
	/**
	 * Tyhjent pakettipuskurin.
	 */
	public void flushPackets() {
		while (!buffer.isEmpty()) {
			buffer.pop();
		}
	}
	
	/**
	 * Kutsuu puskurin push() metodia. Jos puskuri on tysi, niin annettu paketti 
	 * hvitetn. Sen jlkeen kun paketti on listty puskuriin, siirretn 
	 * suoritusvuoro vastaanottavalle reitittimelle. Ainoastaan thn 
	 * vastaanottoporttiin liitetty yksiksitteinen lhetysportti saa kytt 
	 * tt metodia!
	 *
	 * @see PacketBuffer#push(Packet)
	 */
	void push(Packet packet) {
		if (buffer.push(packet)) {
			to.tick();
		} else {
			packet.markAsDropped();
		}
	}
	
	/**
	 * Kutsuu puskurin pop() metodia.
	 *
	 * @see PacketBuffer#pop()
	 */
	public Packet pop() {
		return buffer.pop();
	}
	
	/**
	 * Kutsuu puskurin getFirst() metodia.
	 *
	 * @see PacketBuffer#getFirst()
	 */
	public Packet getFirst() {
		return buffer.getFirst();
	}
	
	/**
	 * Kutsuu puskurin getCount() metodia.
	 *
	 * @see PacketBuffer#getCount()
	 */
	public int getCount() {
		return buffer.getCount();
	}
	
	/**
	 * Kutsuu puskurin isFull() metodia.
	 *
	 * @see PacketBuffer#isFull()
	 */
	public boolean isFull() {
		return buffer.isFull();
	}
	
	/**
	 * Kutsuu puskurin isEmpty() metodia.
	 *
	 * @see PacketBuffer#isEmpty()
	 */
	public boolean isEmpty() {
		return buffer.isEmpty();
	}
	
	/**
	 * Palauttaa portin tilan esityksen muodossa:
	 * recieveBuffer toID
	 */
	public String toString() {
		return buffer.toString()+" "+to.getID();
	}
}
