// Tietorakenteiden harjoitusty (syksy 2004), Esko Luontola
// Tietoliikenneverkon reititysalgoritmien vertailu, "Sim Router"

/**
 * Routereita sisltv prioriteettijono, jossa eniten yhteyksi omistava Router 
 * on ensimmisen.
 */
public class RouterJoinHeap {
	
	/**
	 * Prioriteettijonon toteuttava keko.
	 */
	private Heap heap = new Heap() {
		protected boolean isGreaterOrEqual(int first, int last) {
			return ((Router)get(first)).getConnections() >= ((Router)get(last)).getConnections();
		}
	};
	
	/**
	 * Luo uuden jonon.
	 */
	public RouterJoinHeap() {}
	
	/**
	 * Yhdist indekseiss one ja two olevat reitittimet toisiinsa. Palauttaa
	 * true, jos yhdistys onnistui.
	 */
	public boolean joinRouters(int one, int two) {
		boolean ok = get(one).join(get(two));
		if (ok) {
			int min = Math.min(one, two);
			int max = Math.max(one, two);
			heap.pushUp(max);
			heap.pushUp(min);
		}
		return ok;
	}
	
	/**
	 * Yhdist reitittimen router reitittimeen indeksiss other. Palauttaa
	 * true, jos yhdistys onnistui.
	 */
	public boolean joinRouters(Router router, int other) {
		boolean ok = router.join(get(other));
		if (ok) {
			heap.pushUp(other);
		}
		return ok;
	}
	
	/**
	 * Lis alkion jonon loppuun.
	 */
	public void push(Router obj) {
		heap.push(obj);
	}
	
	/**
	 * Poistaa ensimmisen alkion jonosta.
	 */
	public Router pop() {
		return (Router)heap.pop();
	}
	
	/**
	 * Palauttaa jonon ensimmisen alkion.
	 */
	public Router getFirst() {
		return (Router)heap.getFirst();
	}
	
	/**
	 * Palauttaa annetussa indeksiss olevan alkion.
	 */
	public Router get(int index) {
		return (Router)heap.get(index);
	}
	
	/**
	 * Palauttaa jonon alkioiden lukumrn.
	 */
	public int size() {
		return heap.size();
	}
}
