// Tietorakenteiden harjoitusty (syksy 2004), Esko Luontola
// Tietoliikenneverkon reititysalgoritmien vertailu, "Sim Router"

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;

/**
 * Ohjelman pkyttliittym.
 */
public class SimRouter extends JFrame {
	
	public static boolean DEBUG = false;
	
	private JFormattedTextField routers;
	
	private JFormattedTextField connections;
	
	private JFormattedTextField maxPorts;
	
	private JFormattedTextField bufferSize;
	
	private JFormattedTextField maxTransferDelay;
	
	private JFormattedTextField preTicks;
	
	private String[] protocollData = {"FLOOD", "HOT_POTATO", "SHORTEST_ROUTE", "FASTEST_ROUTE"};
	private JComboBox protocoll;
	
	private JFormattedTextField ttl;
	
	private JFormattedTextField packets;
	
	private JFormattedTextField packetsInterval;
	
	private JFormattedTextField ticks;
	
	private JFormattedTextField repeats;
	
	private JButton makeNetwork;
	
	private JButton resetSimulation;
	
	private JButton runSimulation;
	
	private JTextArea output;
	
	/**
	 * Luo ohjelman pkyttliittymn.
	 */
	public SimRouter() {
		setTitle("Sim Router  /  Copyright \u00a9 2004 Esko Luontola");
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(makeControls(), BorderLayout.NORTH);
		getContentPane().add(makeOutput(), BorderLayout.CENTER);
		getContentPane().add(new ProgressBar(), BorderLayout.SOUTH);
		
		addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
	}
	
	/**
	 * Luo kyttliittymn toimintonapit ja kentt parametrien syttmiseksi.
	 */
	private Component makeControls() {
		JPanel p = new JPanel();
		p.setLayout(new BorderLayout());
		p.add(makeInputFields(), BorderLayout.CENTER);
		p.add(makeButtons(), BorderLayout.SOUTH);
		return p;
	}
	
	/**
	 * Luo kyttliittymn toimintonapit.
	 */
	private Component makeButtons() {
		JPanel p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER));
		
		makeNetwork = new JButton("Create Network");
		resetSimulation = new JButton("Reset Simulation");
		runSimulation = new JButton("Run Simulation");
		
		makeNetwork.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				setAllEnabled(false);
				makeNetwork.setEnabled(false);
				Thread t = new Thread() {
					public void run() {
						makeNetwork();
						resetSimulation.setEnabled(false);
						runSimulation.setEnabled(true);
						setAllEnabled(true);
					}
				};
				t.start();
			}
		});
		
		resetSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				setAllEnabled(false);
				resetSimulation.setEnabled(false);
				Simulation.flushPackets();
				runSimulation.setEnabled(true);
				setAllEnabled(true);
				output.setText("Simulation reseted");
			}
		});
		
		runSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				output.setText("");
				setAllEnabled(false);
				runSimulation.setEnabled(false);
				resetSimulation.setEnabled(false);
				Thread t = new Thread() {
					public void run() {
						runSimulation();
						resetSimulation.setEnabled(true);
						runSimulation.setEnabled(true);
						setAllEnabled(true);
					}
				};
				t.start();
			}
		});
		
		resetSimulation.setEnabled(false);
		runSimulation.setEnabled(false);
		
		p.add(makeNetwork);
		p.add(resetSimulation);
		p.add(runSimulation);
		
		return p;
	}
	
	/**
	 * Luo kyttliittymn kentt parametrien syttmiseksi.
	 */
	private Component makeInputFields() {
		JPanel p = new JPanel();
		p.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints(); 
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.ipadx = 10;
		c.fill = GridBagConstraints.BOTH;
		
		JLabel l;
		JFormattedTextField f;
		int gridx = 0;
		int gridy = 0;
		
		PropertyChangeListener networkChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals("value")) {
					fireNetworkParamsChanged();
				}
			}
		};
		PropertyChangeListener simulationChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals("value")) {
					fireSimulationParamsChanged();
				}
			}
		};
		
		// ROW START
		gridx = 0;
		l = new JLabel("Routers ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(100));
		f.setColumns(7);
		f.addPropertyChangeListener(networkChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		routers = f;
		
		gridx++;
		l = new JLabel("Connections ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(250));
		f.setColumns(7);
		f.addPropertyChangeListener(networkChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		connections = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Max Ports per Router ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(Simulation.getMaxPorts()));
		f.setColumns(7);
		f.addPropertyChangeListener(networkChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		maxPorts = f;
		
		gridx++;
		l = new JLabel("Buffer Size ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(Simulation.getBufferSize()));
		f.setColumns(7);
		f.addPropertyChangeListener(networkChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		bufferSize = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Max Transfer Delay ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(Simulation.getMaxTransferDelay()));
		f.setColumns(7);
		f.addPropertyChangeListener(networkChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		maxTransferDelay = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel(" ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Protocoll ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		protocoll = new JComboBox(protocollData);
		protocoll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fireSimulationParamsChanged();
			}
		});
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(protocoll, c);
		
		gridx++;
		l = new JLabel("TTL ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(64));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		ttl = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Packets ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(100));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		packets = f;
		
		gridx++;
		l = new JLabel("Packets Interval ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(5));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		packetsInterval = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Ticks ", JLabel.RIGHT);
		//l.setBorder(BorderFactory.createEtchedBorder());
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(1000));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		ticks = f;
		
		gridx++;
		l = new JLabel("Repeats ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(10));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		repeats = f;
		
		// ROW START
		gridy++;
		gridx = 0;
		l = new JLabel("Initialization Ticks ", JLabel.RIGHT);
		c.anchor = GridBagConstraints.EAST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(l, c);
		
		gridx++;
		f = new JFormattedTextField();
		f.setValue(new Integer(1000));
		f.setColumns(7);
		f.addPropertyChangeListener(simulationChangeListener);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = gridx;
		c.gridy = gridy;
		p.add(f, c);
		preTicks = f;
		
		return p;
	}
	
	/**
	 * Luo ohjelman tulosteita varten tekstialueen.
	 */
	private Component makeOutput() {
		output = new JTextArea(20, 50);
		output.setLineWrap(false);
		
		JScrollPane scrollPane = new JScrollPane(output,
			JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		return scrollPane;
	}
	
	/**
	 * Suoritetaan, kun jotain verkon rakenteen parametri on muutettu.
	 */
	private void fireNetworkParamsChanged() {
		makeNetwork.setEnabled(true);
		resetSimulation.setEnabled(false);
		runSimulation.setEnabled(false);
	}
	
	/**
	 * Suoritetaan, kun jotain simulaation parametri on muutettu.
	 */
	private void fireSimulationParamsChanged() {
		if (!makeNetwork.isEnabled()) {
			resetSimulation.setEnabled(true);
			runSimulation.setEnabled(false);
		}
	}
	
	/**
	 * Palauttaa tekstikentn sislln kokonaislukuna.
	 */
	private int getParam(JFormattedTextField f) {
		Object value = f.getValue();
		if (value instanceof Integer) {
			return ((Integer)value).intValue();
		} else if (value instanceof Long) {
			return ((Long)value).intValue();
		} else if (value instanceof Double) {
			return ((Double)value).intValue();
		} else {
			return 0;
		}
	}
	
	private int getRouters() {
		return getParam(routers);
	}
	
	private int getConnections() {
		return getParam(connections);
	}
	
	private int getMaxPorts() {
		return getParam(maxPorts);
	}
	
	private int getBufferSize() {
		return getParam(bufferSize);
	}
	
	private int getMaxTransferDelay() {
		return getParam(maxTransferDelay);
	}
	
	private int getPreTicks() {
		return getParam(preTicks);
	}
	
	private int getProtocoll() {
		if (protocoll.getSelectedItem().equals("FLOOD")) {
			return Packet.FLOOD;
		} else if (protocoll.getSelectedItem().equals("HOT_POTATO")) {
			return Packet.HOT_POTATO;
		} else if (protocoll.getSelectedItem().equals("SHORTEST_ROUTE")) {
			return Packet.SHORTEST_ROUTE;
		} else if (protocoll.getSelectedItem().equals("FASTEST_ROUTE")) {
			return Packet.FASTEST_ROUTE;
		} else {
			return -1;
		}
	}
	
	private int getTTL() {
		return getParam(ttl);
	}
	
	private int getPackets() {
		return getParam(packets);
	}
	
	private int getPacketsInterval() {
		return getParam(packetsInterval);
	}
	
	private int getTicks() {
		return getParam(ticks);
	}
	
	private int getRepeats() {
		return getParam(repeats);
	}
	
	/**
	 * Est/sallii kaikkien tekstikenttien muokkaamisen.
	 */
	private void setAllEnabled(boolean enabled) {
		routers.setEnabled(enabled);
		connections.setEnabled(enabled);
		maxPorts.setEnabled(enabled);
		bufferSize.setEnabled(enabled);
		maxTransferDelay.setEnabled(enabled);
		preTicks.setEnabled(enabled);
		protocoll.setEnabled(enabled);
		ttl.setEnabled(enabled);
		packets.setEnabled(enabled);
		packetsInterval.setEnabled(enabled);
		ticks.setEnabled(enabled);
		repeats.setEnabled(enabled);
	}
	
	/**
	 * Luo parametrien mukaisen verkon.
	 */
	private void makeNetwork() {
		output.setText("Building the network...");
		Simulation.setBufferSize(getBufferSize());
		Simulation.setMaxPorts(getMaxPorts());
		Simulation.setMaxTransferDelay(getMaxTransferDelay());
		Simulation.prepareSimulation(getRouters(), getConnections());
		output.setText("");
		printNetwork();
		System.gc();
	}
	
	/**
	 * Suorittaa parametrien mukaisen simulaation.
	 */
	private void runSimulation() {
		if (DEBUG) {
			System.out.println("Simulation started.");
		}
		output.setText("");
		printStatisticsHeaders();
		
		Simulation.runSimulation(
			getProtocoll(), 
			getTTL(), 
			getPackets(), 
			getPacketsInterval(), 
			getPreTicks()
		);
		
		for (int i = getRepeats(); i > 0; i--) {
			Statistics s = Simulation.runSimulation(
				getProtocoll(), 
				getTTL(), 
				getPackets(), 
				getPacketsInterval(), 
				getTicks()
			);
			printStatistics(s);
		}
		System.gc();
		if (DEBUG) {
			System.out.println("Simulation ended.");
		}
	}
	
	/**
	 * Kirjoittaa tulosikkunaan tiedot verkon rakenteesta.
	 */
	private void printNetwork() {
		int routers = Simulation.getRouters();
		int connections = Simulation.getConnections();
		output.append("Network has "+routers+" routers and "+connections+" connections.\n");
	}
	
	/**
	 * Kirjoittaa tulosikkunaan tilastotietojen otsikot.
	 */
	private void printStatisticsHeaders() {
		
		output.append("Routers:\t"+Simulation.getRouters()+"\n");
		output.append("Connections:\t"+Simulation.getConnections()+"\n");
		output.append("Max Ports/Router:\t"+Simulation.getMaxPorts()+"\n");
		output.append("Max Transfer Delay:\t"+Simulation.getMaxTransferDelay()+"\n");
		output.append("Buffer Size:\t"+Simulation.getBufferSize()+"\n");
		output.append("\n");
		
		output.append("Protocoll:\t");
		switch (getProtocoll()) {
			case Packet.FLOOD:
				output.append("FLOOD\n");
				break;
			case Packet.HOT_POTATO:
				output.append("HOT_POTATO\n");
				break;
			case Packet.SHORTEST_ROUTE:
				output.append("SHORTEST_ROUTE\n");
				break;
			case Packet.FASTEST_ROUTE:
				output.append("FASTEST_ROUTE\n");
				break;
			default:
				output.append("UNKNOWN\n");
				break;
		}
		output.append("TTL:\t"+getTTL()+"\n");
		output.append("\n");
		
		output.append("Ticks\t");
		output.append("Sent\t");
		output.append("Recieved\t");
		output.append("Dropped\t");
		output.append("Loss\t");
		output.append("MinTime\t");
		output.append("AvgTime\t");
		output.append("MaxTime\t");
		output.append("MinJumps\t");
		output.append("AvgJumps\t");
		output.append("MaxJumps\n");
	}
	
	/**
	 * Kirjoittaa tulosikkunaan tilastotiedot.
	 */
	private void printStatistics(Statistics stats) {
		output.append(stats.getLength()+"\t");
		output.append(stats.getPacketsSent()+"\t");
		output.append(stats.getPacketsRecieved()+"\t");
		output.append(stats.getPacketsDropped()+"\t");
		output.append(Math.round(stats.getPacketLoss()*1000)/1000.0+"\t");
		output.append(stats.getMinTransferTime()+"\t");
		output.append(Math.round(stats.getAvgTransferTime()*1000)/1000.0+"\t");
		output.append(stats.getMaxTransferTime()+"\t");
		output.append(stats.getMinJumps()+"\t");
		output.append(Math.round(stats.getAvgJumps()*1000)/1000.0+"\t");
		output.append(stats.getMaxJumps()+"\n");
		
		if (DEBUG) {
			System.out.println(output.getText());
		}
	}
	
	/**
	 * Kynnist ohjelman.
	 */
	public static void main(String[] args){
		if (args.length > 0 && args[0].equals("DEBUG")) {
			SimRouter.DEBUG = true;
			System.out.println("Program started in debugging state.");
		}
		SimRouter frame = new SimRouter();
		frame.pack();
		Rectangle b = frame.getBounds();
		b.setLocation(200, 200);
		frame.setBounds(b);
		frame.setVisible(true);
	}
}
