// Tietorakenteiden harjoitusty (syksy 2004), Esko Luontola
// Tietoliikenneverkon reititysalgoritmien vertailu, "Sim Router"

/**
 * Pyritt simulaatiota antamalla kullekin komponentille suoritusvuoron ajallaan.
 */
public class Ticker {
	
	/**
	 * Aikayksikk simulaation alusta.
	 */
	private static int tick = 0;
	
	/**
	 * Prioriteettijono, jossa pienimmn timeoutin omaava SendPort on ensimmisen.
	 */
	private static TimeoutQueue timeouts = new TimeoutQueue();
	
	private Ticker() {}
	
	/**
	 * Nollaa ajastimen ja timeoutit.
	 */
	public static void reset() {
		tick = 0;
		timeouts = new TimeoutQueue();
	}
	
	/**
	 * Palauttaa nykyisen ajanhetken aikayksikkin simulaation alusta.
	 */
	public static int getTime() {
		return tick;
	}
	
	/**
	 * Palauttaa pienimmn timeoutin tai -1, jos yksikn SendPort ei odota timeouttia.
	 */
	public static int getNextTimeout() {
		if (timeouts.size() > 0) {
			return timeouts.getFirst().getTimeout();
		} else {
			return -1;
		}
	}
	
	/**
	 * Asettaa ajastimen kutsumaan lhetysporttia port sitten, kun waitTicks 
	 * aikayksikk on kulunut. Palauttaa ajanhetken (simulaation alusta),
	 * jolloin ajastin tulee tyteen.
	 */
	public static void setTimeout(SendPort port, int delay) {
		port.setTimeout(tick + delay);
		timeouts.push(port);
	}
	
	/**
	 * Pyritt simulatiota yhden aikayksikn eteenpin. Hertt vuorossa
	 * olevat SendPortit.
	 */
	public static void tick() {
		tick(1);
	}
	
	/**
	 * Pyritt simulatiota annetun mrn aikayksikkj eteenpin. Hertt
	 * vuorossa olevat SendPortit.
	 */
	public static void tick(int count) {
		while (count > 0) {
			int nextTimeout = getNextTimeout();
			
			if (nextTimeout < 0 || nextTimeout - tick >= count) {
				// kenellekn ei tarvitse antaa vuoroa
				tick += count;
				count = 0;
			} else {
				// anna jonossa ensimmisille vuoro
				count -= nextTimeout - tick;
				tick = nextTimeout;
				while (timeouts.size() > 0 && timeouts.getFirst().getTimeout() <= tick) {
					SendPort port = timeouts.pop();
					port.tick(tick);
				}
			}
		}
	}
	
	/**
	 * Pyritt simulaatiota eteenpin, kunnes verkossa ei en ole liikkuvia 
	 * paketteja. Tt kytetn reitityspakettien lhettmiseen ennen 
	 * simulaation alkua.
	 */
	public static void tickUntilStopped() {
		while (timeouts.size() > 0) {
			tick(1000);
		}
	}
}
